use std::sync::Arc;

use actix::{Actor, StreamHandler};
use actix_web::{Error, HttpRequest, HttpResponse, web};
use actix_web_actors::ws;

use crate::args::Args;

/// Define HTTP actor
struct RelayWS;

impl Actor for RelayWS {
    type Context = ws::WebsocketContext<Self>;
}

/// Handler for ws::Message message
impl StreamHandler<Result<ws::Message, ws::ProtocolError>> for RelayWS {
    fn handle(&mut self, msg: Result<ws::Message, ws::ProtocolError>, ctx: &mut Self::Context) {
        match msg {
            Ok(ws::Message::Ping(msg)) => ctx.pong(&msg),
            Ok(ws::Message::Text(text)) => ctx.text(text),
            Ok(ws::Message::Binary(bin)) => ctx.binary(bin),
            _ => (),
        }
    }
}

#[derive(serde::Deserialize)]
pub struct WebSocketQuery {
    id: usize,
    token: String,
}

pub async fn relay_ws(req: HttpRequest, stream: web::Payload,
                      query: web::Query<WebSocketQuery>,
                      conf: web::Data<Arc<Args>>) -> Result<HttpResponse, Error> {
    if !conf.tokens.contains(&query.token) {
        log::error!("Rejected WS request from {:?} due to invalid token!", req.peer_addr());
        return Ok(HttpResponse::Unauthorized().json("Invalid / missing token!"));
    }

    if conf.ports.len() <= query.id {
        log::error!("Rejected WS request from {:?} due to invalid port number!", req.peer_addr());
        return Ok(HttpResponse::BadRequest().json("Invalid port number!"));
    }

    let upstream_addr = format!("{}:{}", conf.upstream_server, conf.ports[query.id]);

    let resp = ws::start(RelayWS {}, &req, stream);
    log::info!("Opening new WS connection for {:?} to {}", req.peer_addr(), upstream_addr);
    resp
}